package com.dzeio.openhealth.ui.home

import android.animation.ValueAnimator
import android.content.SharedPreferences
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.RectF
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import androidx.preference.PreferenceManager
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.databinding.FragmentHomeBinding
import com.dzeio.openhealth.graphs.WeightChart
import com.dzeio.openhealth.ui.weight.AddWeightDialog
import com.dzeio.openhealth.units.UnitFactory
import com.dzeio.openhealth.utils.DrawUtils
import com.dzeio.openhealth.utils.GraphUtils
import com.google.android.material.color.MaterialColors
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collectLatest
import kotlin.math.max
import kotlin.math.min

@AndroidEntryPoint
class HomeFragment : BaseFragment<HomeViewModel, FragmentHomeBinding>(HomeViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentHomeBinding
        get() = FragmentHomeBinding::inflate

    private val settings: SharedPreferences by lazy {
        PreferenceManager.getDefaultSharedPreferences(requireContext())
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // Bindings
        binding.addWeight.setOnClickListener {
            AddWeightDialog().show(requireActivity().supportFragmentManager, null)
        }

        binding.fragmentHomeWaterAdd.setOnClickListener {
            val water = viewModel.water.value
            if (water == null || !water.isToday()) {
                val w = Water()
                w.value = viewModel.waterCupSize
                viewModel.updateWater(w)
            } else {
                water.value += viewModel.waterCupSize
                viewModel.updateWater(water)
            }
        }

        binding.fragmentHomeWaterTotal.text =
            String.format(
                resources.getString(viewModel.waterUnit.unit),
                viewModel.dailyWaterIntake
            )

        binding.fragmentHomeWaterRemove.setOnClickListener {
            val water = viewModel.water.value
            if (water != null) {
                water.value -= viewModel.waterCupSize
                if (water.value <= 0) {
                    viewModel.deleteWater(water)
                } else {
                    viewModel.updateWater(water)
                }
            }
        }

        binding.listWeight.setOnClickListener {
            findNavController().navigate(HomeFragmentDirections.actionNavHomeToNavListWeight())
        }

        binding.gotoWaterHome.setOnClickListener {
            findNavController().navigate(HomeFragmentDirections.actionNavHomeToNavWaterHome())
        }

        GraphUtils.lineChartSetup(
            binding.weightGraph,
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorPrimary
            ),
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorOnBackground
            )
        )
    }

    private fun updateGraph(list: List<Weight>) {
        WeightChart.setup(
            binding.weightGraph,
            requireView(),
            list,
            viewModel.weightUnit,
            viewModel.goalWeight?.toFloat()
        )

//                legend.apply {
//                    isEnabled = true
//                    form = Legend.LegendForm.LINE
//
//                    if (goal != null) {
//                        val legendEntry = LegendEntry().apply {
//                            label = "Weight Goal"
//                            formColor = Color.RED
//                        }
//                        setCustom(arrayOf(legendEntry))
//                    }
//                }
    }

    override fun onStart() {
        super.onStart()

        lifecycleScope.launchWhenStarted {
            viewModel.fetchWeights().collectLatest {
                updateGraph(it)
            }
            updateWater(0)
        }

        viewModel.water.observe(viewLifecycleOwner) {
            if (it != null) {
                updateWater(it.value)
            } else {
                updateWater(0)
            }
        }
    }

    private var oldValue = 0f

    private fun updateWater(newValue: Int) {

        val waterUnit =
            UnitFactory.volume(settings.getString("water_unit", "milliliter") ?: "Milliliter")

        binding.fragmentHomeWaterCurrent.text =
            String.format(
                resources.getString(waterUnit.unit),
                (newValue * waterUnit.modifier).toInt()
            )

        var width = 1500
        var height = 750

        if (binding.background.width != 0) {
            width = binding.background.width
            height = binding.background.height
        }

        val graph = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888)

        val canvas = Canvas(graph)
        val rect = RectF(
            10f,
            15f,
            90f,
            85f
        )

//        DrawUtils.drawRect(
//            canvas,
//            RectF(
//                0f,
//                0f,
//                100f,
//                100f
//            ),
//            MaterialColors.getColor(
//                requireView(),
//                com.google.android.material.R.attr.colorOnPrimary
//            ),
//            3f
//        )

        DrawUtils.drawArc(
            canvas,
            100f,
            rect,
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorOnPrimary
            ),
            3f
        )

        val animator = ValueAnimator.ofInt(
            min(this.oldValue, viewModel.dailyWaterIntake.toFloat()).toInt(),
            min(newValue, viewModel.dailyWaterIntake)
        )
        animator.duration = 300 // ms
        animator.addUpdateListener {

            this.oldValue = 100 * it.animatedValue as Int / viewModel.dailyWaterIntake.toFloat()
//            Log.d("Test2", "${this.oldValue}")

            DrawUtils.drawArc(
                canvas,
                max(this.oldValue, 1f),
                rect,
                MaterialColors.getColor(
                    requireView(),
                    com.google.android.material.R.attr.colorPrimary
                ),
                6f
            )
            canvas.save()
            binding.background.setImageBitmap(graph)
        }
        animator.start()
    }
}
